use std::collections::BTreeSet;

use crate::util::{index::*, parse::ParseExt};

fn parse(input: &str) -> impl Iterator<Item = (u8, u32)> + '_ {
    let dirs = input.bytes().filter(u8::is_ascii_uppercase);
    let steps: Vec<_> = input.u32s().collect();

    dirs.zip(steps)
}

pub fn part1(input: &str) -> impl std::fmt::Display {
    let mut pos = Ix2::ORIGIN;
    let mut dir = Ix2::UP;

    for (d, s) in parse(input) {
        dir = if d == b'R' {
            dir.clockwise()
        } else {
            dir.counter_clockwise()
        };

        pos += dir * (s as i64);
    }

    pos.manhatten(Ix2::ORIGIN)
}

pub fn part2(input: &str) -> impl std::fmt::Display {
    let mut pos = Ix2::ORIGIN;
    let mut dir = Ix2::UP;
    let mut set = BTreeSet::new();

    for (d, s) in parse(input) {
        dir = if d == b'R' {
            dir.clockwise()
        } else {
            dir.counter_clockwise()
        };

        for _ in 0..s {
            pos += dir;
            if !set.insert(pos) {
                return pos.manhatten(Ix2::ORIGIN);
            }
        }
    }
    unreachable!()
}

#[test]
fn test_part1() {
    assert_eq!("12", part1("R5, L5, R5, R3").to_string())
}

#[test]
fn test_part2() {
    assert_eq!("4", part2("R8, R4, R4, R8").to_string())
}
