use crate::util::parse::ParseExt;

fn solve(input: &str, filter: fn(u32, u32, u32, u32) -> bool) -> usize {
    input
        .lines()
        .filter_map(|line| {
            let mut nums = line.u32s();

            if (filter)(nums.next()?, nums.next()?, nums.next()?, nums.next()?) {
                Some(())
            } else {
                None
            }
        })
        .count()
}

pub fn part1(input: &str) -> impl std::fmt::Display {
    solve(input, |s1, e1, s2, e2| {
        s1 <= s2 && e1 >= e2 || s1 >= s2 && e1 <= e2
    })
}

pub fn part2(input: &str) -> impl std::fmt::Display {
    solve(input, |s1, e1, s2, e2| {
        s1 >= s2 && s1 <= e2 || s2 >= s1 && s2 <= e1
    })
}

#[test]
fn test_part1() {
    assert_eq!(
        "2",
        part1("2-4,6-8\n2-3,4-5\n5-7,7-9\n2-8,3-7\n6-6,4-6\n2-6,4-8").to_string()
    )
}

#[test]
fn test_part2() {
    assert_eq!(
        "4",
        part2("2-4,6-8\n2-3,4-5\n5-7,7-9\n2-8,3-7\n6-6,4-6\n2-6,4-8").to_string()
    )
}
