use crate::util::parse::ParseExt;

fn parse(input: &str) -> Option<(Vec<Vec<char>>, impl Iterator<Item = [u32; 3]> + '_)> {
    let (board, moveset) = input.split_once("\n\n")?;
    let size = board.lines().last()?.u32s().count();
    let mut crates: Vec<Vec<char>> = vec![vec![]; size + 1];

    board.lines().for_each(|line| {
        if !line.contains('[') {
            return;
        }

        line.chars()
            .enumerate()
            .filter(|(_idx, c)| c.is_ascii_uppercase())
            .for_each(|(idx, c)| {
                crates[((idx + 2) / 4) + 1].push(c);
            });
    });
    for v in crates.iter_mut() {
        v.reverse();
    }

    let moves = moveset.lines().filter_map(|line| {
        let mut nums = line.u32s();
        Some([nums.next()?, nums.next()?, nums.next()?])
    });

    Some((crates, moves))
}

pub fn part1(input: &str) -> impl std::fmt::Display {
    let (mut crates, moves) = parse(input).expect("parsing failed");
    let mut out: Vec<String> = vec![String::new(); crates.len()];

    for m in moves {
        let mut items = {
            let mut items = Vec::with_capacity(m[0] as usize);
            let from: &mut Vec<char> = crates[m[1] as usize].as_mut();
            for _ in 0..m[0] {
                items.push(from.pop().expect("AOC please don't hate us"));
            }
            items
        };
        crates[m[2] as usize].append(&mut items);
    }

    for v in crates.iter_mut().skip(1) {
        out.push(v.pop().expect("yes").to_string());
    }

    out.join("")
}

pub fn part2(input: &str) -> impl std::fmt::Display {
    let (mut crates, moves) = parse(input).expect("parsing failed");
    let mut out: Vec<String> = vec![String::new(); crates.len()];

    for m in moves {
        let mut items = {
            let mut items = Vec::with_capacity(m[0] as usize);
            let from: &mut Vec<char> = crates[m[1] as usize].as_mut();
            for _ in 0..m[0] {
                items.push(from.pop().expect("AOC please don't hate us"));
            }
            items.reverse();
            items
        };
        crates[m[2] as usize].append(&mut items);
    }

    for v in crates.iter_mut().skip(1) {
        out.push(v.pop().expect("yes").to_string());
    }

    out.join("")
}

#[test]
fn test_part1() {
    assert_eq!(
        "2",
        part1("2-4,6-8\n2-3,4-5\n5-7,7-9\n2-8,3-7\n6-6,4-6\n2-6,4-8").to_string()
    )
}

#[test]
fn test_part2() {
    assert_eq!(
        "4",
        part2("2-4,6-8\n2-3,4-5\n5-7,7-9\n2-8,3-7\n6-6,4-6\n2-6,4-8").to_string()
    )
}
