static NUMS: &[&str] = &[
    "one", "two", "three", "four", "five", "six", "seven", "eight", "nine",
];

pub fn part1(input: &str) -> impl std::fmt::Display {
    input
        .lines()
        .map(|l| {
            let nums = l
                .chars()
                .filter_map(|c| c.is_ascii_digit().then_some(c as u8 - b'0'))
                .collect::<Vec<_>>();
            let a = nums.first().unwrap();
            let b = nums.last().unwrap();

            (a * 10 + b) as u32
        })
        .sum::<u32>()
}

pub fn part2(input: &str) -> impl std::fmt::Display {
    input
        .lines()
        .map(|l| {
            let mut nums = vec![];
            for i in 0..l.len() {
                let first = l[i..i + 1].chars().next().unwrap();
                if first.is_ascii_digit() {
                    nums.push((first as u8 - b'0') as u32);
                }
                for (j, e) in NUMS.iter().enumerate() {
                    if l[i..].starts_with(e) {
                        nums.push(j as u32 + 1);
                    }
                }
            }
            let a = nums.first().unwrap();
            let b = nums.last().unwrap();

            a * 10 + b
        })
        .sum::<u32>()
}

#[test]
fn test_part1() {
    assert_eq!(
        "142",
        part1("1abc2\npqr3stu8vwx\na1b2c3d4e5f\ntreb7uchet").to_string()
    )
}

#[test]
fn test_part2() {
    assert_eq!(
        "45000",
        part2("1000\n2000\n3000\n\n4000\n\n5000\n6000\n\n7000\n8000\n9000\n\n10000").to_string()
    )
}
